use 5.010;
use strict;
use warnings;
use Test::More tests => 2;
use Net::Statsd;

ok(1, "Net::Statsd module loaded");

my $version = Net::Statsd->VERSION();
ok($version, "Net::Statsd version is $version");

$Net::Statsd::HOST = 'localhost';
$Net::Statsd::PORT = 8125;

say Net::Statsd::increment('test.statsd');
